<?php

namespace Drupal\ercore_core;

use Drupal\ercore\ErcoreStartDate;

/**
 * Class ErcoreOutputsBuild.
 *
 * @package Drupal\ercore_core
 */
class ErcoreOutputsExpanded {

  /**
   * Builds data array for participant data block.
   *
   * @return array
   *   Returns array of objects.
   */
  public static function buildDataArray() {
    return [
      'patents' => [
        'awarded' => [
          'current' => 0,
          'cumulative' => 0,
        ],
        'pending' => [
          'current' => 0,
          'cumulative' => 0,
        ],
        'licensed' => [
          'current' => 0,
          'cumulative' => 0,
        ],
      ],
      'proposals' => [
        'submitted' => [
          'current' => [
            'number' => 0,
            'funds' => 0,
          ],
          'cumulative' => [
            'number' => 0,
            'funds' => 0,
          ],
        ],
        'awarded' => [
          'current' => [
            'number' => 0,
            'funds' => 0,
            'awarded_funds' => 0,
          ],
          'cumulative' => [
            'number' => 0,
            'funds' => 0,
            'awarded_funds' => 0,
          ],
        ],
        'pending' => [
          'current' => [
            'number' => 0,
            'funds' => 0,
          ],
          'cumulative' => [
            'number' => 0,
            'funds' => 0,
          ],
        ],
      ],
      'publications' => [
        'primary' => [
          'current' => 0,
          'cumulative' => 0,
        ],
        'partial' => [
          'current' => 0,
          'cumulative' => 0,
        ],
      ],
      'hired' => [
        'male' => [
          'current' => 0,
          'cumulative' => 0,
        ],
        'female' => [
          'current' => 0,
          'cumulative' => 0,
        ],
        'minority' => [
          'current' => 0,
          'cumulative' => 0,
        ],
        'disabled' => [
          'current' => 0,
          'cumulative' => 0,
        ],
      ],
      'post-doc' => [
        'male' => [
          'current' => 0,
          'cumulative' => 0,
        ],
        'female' => [
          'current' => 0,
          'cumulative' => 0,
        ],
        'minority' => [
          'current' => 0,
          'cumulative' => 0,
        ],
        'disabled' => [
          'current' => 0,
          'cumulative' => 0,
        ],
      ],
      'graduate' => [
        'male' => [
          'current' => 0,
          'cumulative' => 0,
        ],
        'female' => [
          'current' => 0,
          'cumulative' => 0,
        ],
        'minority' => [
          'current' => 0,
          'cumulative' => 0,
        ],
        'disabled' => [
          'current' => 0,
          'cumulative' => 0,
        ],
      ],
      'undergraduate' => [
        'male' => [
          'current' => 0,
          'cumulative' => 0,
        ],
        'female' => [
          'current' => 0,
          'cumulative' => 0,
        ],
        'minority' => [
          'current' => 0,
          'cumulative' => 0,
        ],
        'disabled' => [
          'current' => 0,
          'cumulative' => 0,
        ],
      ],
    ];
  }

  /**
   * Build array of IDs.
   *
   * @param string $type
   *   Type of node to filter by.
   *
   * @return array
   *   Array of User IDs.
   */
  public static function getNodeIds($type) {
    $query = \Drupal::entityQuery('node')
      ->condition('type', $type)
      ->condition('status', 1)
      ->accessCheck(TRUE); // Explicitly enable access checking;
    return $query->execute();
  }

  /**
   * Build patents data.
   *
   * @return object
   *   Object with patent type as key and array of nodes as the value.
   */
  public static function getPatents() {
    $ids = self::getNodeIds('ercore_patent');
    $nodes = [
        'licensed' => [],
        'awarded' => [],
        'pending' => [],
    ];
    foreach ($ids as $id) {
      $licensed = '';
      $pending = '';
      $awarded = '';
      $node = \Drupal::entityTypeManager()->getStorage('node')->load($id);
      if (!$node->get('field_ercore_pt_license')->isEmpty()) {
        $nodes['licensed'][$id] = $node->get('title')->value;
      }
      else if (!$node->get('field_ercore_pt_award')->isEmpty()) {
        $nodes['awarded'][$id] = $node->get('title')->value;
      }
      else if (!$node->get('field_ercore_pt_provisional')->isEmpty()) {
        $nodes['pending'][$id] = $node->get('title')->value;
      }
    }
    // dpr($nodes);
    return $nodes;
  }

  /**
   * Build proposals data.
   *
   * @return array
   *   Array of node data.
   */
  public static function getProposals() {
    $ids = self::getNodeIds('ercore_proposal');
    $nodes = [
      'submitted' => [],
      'awarded' => [],
      'pending' => [],
    ];
    foreach ($ids as $id) {
      $node = \Drupal::entityTypeManager()->getStorage('node')->load($id);
      //this is how you get the status of the proposals
      $status = $node->get('field_ercore_pp_proposal_status')->value;
      // dpm($status);
      
      if (!$node->get('field_ercore_pp_proposal_submit')->isEmpty()) {
        // $submitted_var = $node->get('field_ercore_pp_proposal_submit')->value;
        $nodes['submitted'][$id] = $node->get('title')->value;
      }
      $award_status = [ //this is array that is checked to see if it contains status
        'Awarded',
        'Expired',
      ];
      if (in_array($status, $award_status)) {
        $nodes['awarded'][$id] = $node->get('title')->value;
      }
      $pending_status = [

        'Pending',
        'Submitted', // added this to be part of pending status as there has not been another status given to it yet

      ];
      if (in_array($status, $pending_status)) {
        $nodes['pending'][$id] = $node->get('title')->value;
      }
    }
    // dpm($nodes);
    return $nodes;
  }

  /**
   * Build publications data.
   *
   * @return array
   *   Array of node data.
   */
  public static function getPublications() {
    $ids = self::getNodeIds('ercore_publication');
    $nodes = [];
    $published = '';
    $rii = '';
    foreach ($ids as $id) {
      $node = \Drupal::entityTypeManager()->getStorage('node')->load($id);
      if (!$node->get('field_ercore_pb_date')->isEmpty()) {
        $published_var = $node->get('field_ercore_pb_date')->value;
        $published = ErcoreStartDate::dateArgumentToUnix($published_var);
        if (!$node->get('field_ercore_pb_rii_support')->isEmpty()) {
          $rii = $node->get('field_ercore_pb_rii_support')->value;
        }
        $title = $node->get('title')->value;
        $nodes[] = [
          'rii' => $rii,
          'published' => $published,
          'id' => $id,
          'title' => $title,
        ];
      }
    }
    return $nodes;
  }

  /**
   * Build Patents object.
   *
   *
   * @return array
   *   Array of User IDs.
   */
  public static function filteredPatents($cumulative = FALSE) {

    $nodes = self::getPatents();
    $filtered_nodes = [
      'licensed' => [],
      'awarded' => [],
      'pending' => [],
    ];
    if (!empty($nodes['licensed'])) {
      foreach ($nodes['licensed'] as $id => $name){
        $node = \Drupal::entityTypeManager()->getStorage('node')->load($id);
        if (!$node->get('field_ercore_pt_license')->isEmpty()) {
          $licensed_var = $node->get('field_ercore_pt_license')->value;
          $licensed = ErcoreStartDate::dateArgumentToUnix($licensed_var);
          if(self::outputDateFilter($licensed, $cumulative)){
            $filtered_nodes['licensed'][$id] = $name;
          }
        }
      }
      // $data['patents']['licensed']['current'] += self::outputDateFilter($node['licensed'], FALSE);
    }
    if (!empty($nodes['awarded'])) {
      foreach ($nodes['awarded'] as $id => $name){
        $node = \Drupal::entityTypeManager()->getStorage('node')->load($id);
        if (!$node->get('field_ercore_pt_award')->isEmpty()) {
          $awarded_var = $node->get('field_ercore_pt_award')->value;
          $awarded = ErcoreStartDate::dateArgumentToUnix($awarded_var);
          if(self::outputDateFilter($awarded, $cumulative)){
              $filtered_nodes['awarded'][$id] = $name;
          }
        }
      }
      // $data['patents']['awarded']['current'] += self::outputDateFilter($node['awarded'], FALSE);
    }
    if (!empty($nodes['pending'])) {
      foreach ($nodes['pending'] as $id => $name){
        $node = \Drupal::entityTypeManager()->getStorage('node')->load($id);
        if (!$node->get('field_ercore_pt_provisional')->isEmpty()) {
          $pending_var = $node->get('field_ercore_pt_provisional')->value;
          $pending = ErcoreStartDate::dateArgumentToUnix($pending_var);
          if(self::outputDateFilter($pending, $cumulative)){
            $filtered_nodes['pending'][$id] = $name;
          }
        }
      }
      // $data['patents']['pending']['current'] += self::outputDateFilter($node['pending'], FALSE);
    }

    return $filtered_nodes;
  }

  /**
   * Build Proposal object.
   *
   *  @param boolean $cumulative
   *   Boolean to identify period or cumulative date range.
   *
   * @return array
   *   Array of filtered proposal node ids
   */
  public static function filteredProposals($cumulative = FALSE) {
    $nodes = self::getProposals();
    $filtered_nodes = [
      'submitted' => [],
      'awarded' => [],
      'pending' => [],
    ];
    foreach ($nodes as $type => $arr) {
      // dpm($type, '$type');
      // dpm($arr, '$arr');
      foreach($arr as $id => $name){
        // dpm($id);
        // dpm($name);
        $node = \Drupal::entityTypeManager()->getStorage('node')->load($id);
        
        if( $type == "submitted"){
          if (!$node->get('field_ercore_pp_proposal_submit')->isEmpty()) {
            $submitted_var = $node->get('field_ercore_pp_proposal_submit')->value;
            $submitted = ErcoreStartDate::dateArgumentToUnix($submitted_var);
            if(self::outputDateFilter($submitted, $cumulative)){
              // dpm('Accepted');
              $filtered_nodes['submitted'][$id] = $name;
            }
          }
        }
        else if( $type == "awarded"){
          if (!$node->get('field_ercore_pp_award_start')->isEmpty()) {
            $awarded = [];
            if (!$node->get('field_ercore_pp_award_start')->isEmpty()) {
              $start = $node->get('field_ercore_pp_award_start')->value;
              $awarded['start'] = ErcoreStartDate::dateArgumentToUnix($start);
            }
            if (!$node->get('field_ercore_pp_award_end')->isEmpty()) {
              $end = $node->get('field_ercore_pp_award_end')->value;
              $awarded['end'] = ErcoreStartDate::dateArgumentToUnix($end);
            }
            // $start = $node->get('field_ercore_pp_award_start')->value;
            // $awarded['start'] = ErcoreStartDate::dateArgumentToUnix($start);
            // $end = $node->get('field_ercore_pp_award_end')->value;
            // $awarded['end'] = ErcoreStartDate::dateArgumentToUnix($end);
            if(self::outputDateRangeFilter($awarded, $cumulative)){
              // dpm('Accepted');
              $filtered_nodes['awarded'][$id] = $name;
            }
          }
        }
        else if( $type == "pending"){
          if (!$node->get('field_ercore_pp_proposal_pending')->isEmpty()) {
            $pending_var = $node->get('field_ercore_pp_proposal_pending')->value;
            $pending = ErcoreStartDate::dateArgumentToUnix($pending_var);
            if(self::outputDateFilter($pending, $cumulative)){
              $filtered_nodes['pending'][$id] = $name;
            }
          }
        }
      }
    }
    return $filtered_nodes;
  }

  /**
   * Build Publications object.
   *
   *  @param boolean $cumulative
   *   Boolean to identify period or cumulative date range.
   *
   * @return array
   *   Array of User IDs.
   */
  public static function filteredPublications($cumulative = FALSE) {
    $nodes = self::getPublications();
    foreach ($nodes as $node) {
      $title = $node['title'];
      $id = $node['id'];
      if ($node['rii'] === '0') {
        if(self::outputDateFilter($node['published'], $cumulative)){
          $filtered_nodes['primary'][$id] = $title;
        }
        // $data['publications']['primary']['current'] += self::outputDateFilter($node['published'], FALSE);
        // $data['publications']['primary']['cumulative'] += self::outputDateFilter($node['published'], TRUE);
      }
      if ($node['rii'] === '1') {
        if(self::outputDateFilter($node['published'], $cumulative)){
          $filtered_nodes['partial'][$id] = $title;
        }
        // $data['publications']['partial']['current'] += self::outputDateFilter($node['published'], FALSE);
        // $data['publications']['partial']['cumulative'] += self::outputDateFilter($node['published'], TRUE);
      }
    }
    return $filtered_nodes;
  }

  /**
   * Build Publications object.
   *
   *  @param boolean $cumulative
   *   Boolean to identify period or cumulative date range.
   * @param string $type
   *   Date value used to separate by graph.
   *
   * @return array
   *   Array of User IDs.
   */
  public static function filteredUsers($cumulative, $type) {
    $data =[];
    $data[$type] = [];
    $users = ErcoreParticipantBuild::getUsers();
    foreach ($users as $user) {
      $filter_date = NULL;
      $dates = NULL;
      $count = FALSE;
      $count_c = FALSE;
      if ($type === 'post-doc') {
        if ($user['role'] === 'post-doc') {
          $dates = [
            'start' => $user['start'],
            'end' => $user['end'],
          ];
          if (self::outputDateRangeFilter($dates, FALSE) == 1) {
            $count = TRUE;
          }
          if (self::outputDateRangeFilter($dates, TRUE) == 1) {
            $count_c = TRUE;
          }
        }
      }
      else {
        if ($type === 'undergraduate' && $user['role'] === 'undergraduate') {
          $filter_date = $user[$type];
        }
        elseif ($type === 'hired' && ($user['role'] === 'faculty' || $user['role'] === 'post-doc')) {
          $filter_date = $user[$type];
        }
        elseif ($type === 'graduate' && ($user['role'] === 'graduate')) {
          $filter_date = $user['masters'];
          if (!empty($user['doctoral'])) {
            $filter_date = $user['doctoral'];
          }
        }
        if (!empty($filter_date)) {
          if (self::outputDateFilter($filter_date, FALSE) == 1) {
            $count = TRUE;
          }
          if (self::outputDateFilter($filter_date, TRUE) == 1) {
            $count_c = TRUE;
          }
        }
      }
      if ($count === TRUE || $count_c === TRUE) {
        $user_status = [
          'type' => $type,
          'name' => $user['name'],
          'id' => $user['id'],
          'current' => $count,
          'cumulative' => $count_c,
          'Male' => FALSE,
          'Female' => FALSE,
          'minority' => FALSE,
          'disabled' => FALSE,
        ];
        $in = [
          'American Indian or Alaskan Native',
          'Black or African American',
          'Pacific Islander',
          'Native Hawaiian',
        ];
        if (!empty($user['race']) && in_array($user['race'], $in)) {
          $user_status['minority'] = TRUE;
        }
        if (!empty($user['ethnicity']) && $user['ethnicity'] === 'Hispanic or Latino') {
          $user_status['minority'] = TRUE;
        }
        if (!empty($user['gender'])) {
          $user_status[$user['gender']] = TRUE;
        }
        if (!empty($user['disability']) && $user['disability'] !== 'None') {
          $user_status['disabled'] = TRUE;
        }
        // $data[] = $user_status;
        // $user_node = \Drupal::entityTypeManager()->getStorage('user')->load($user["id"]);
        // dpm($user_node);
        $data[$type] = self::countUsers($data[$type], $user_status);
      }
    }
    return $data;
  }

  /**
   * Increment (count) users in data sub-arrays.
   *
   * @param array $typeArray
   *   Data array capturing data.
   * @param array $user
   *   User data array to use for counts.
   *
   * @return array
   *   Return modified array.
   */
  public static function countUsers(array $typeArray, array $user) {
    if ($user['current'] === TRUE) {
      if ($user['Male'] === TRUE) {
        $typeArray['male']['current'][] = $user;
      }
      elseif ($user['Female'] === TRUE) {
        $typeArray['female']['current'][] = $user;
      }
      if ($user['minority'] === TRUE) {
        $typeArray['minority']['current'][] = $user;
      }
      if ($user['disabled'] === TRUE) {
        $typeArray['disabled']['current'][] = $user;
      }
    }
    if ($user['cumulative'] === TRUE) {
      if ($user['Male'] === TRUE) {
        $typeArray['male']['cumulative'][] = $user;
      }
      elseif ($user['Female'] === TRUE) {
        $typeArray['female']['cumulative'][] = $user;
      }
      if ($user['minority'] === TRUE) {
        $typeArray['minority']['cumulative'][] = $user;
      }
      if ($user['disabled'] === TRUE) {
        $typeArray['disabled']['cumulative'][] = $user;
      }
    }
    return $typeArray;
  }

  /**
   * Filter Output Dates.
   *
   * @param int $date
   *   Date to be verified.
   * @param bool $cumulative
   *   Use cumulative dates or not.
   *
   * @return int
   *   Return status of date in range.
   */
  public static function outputDateFilter($date, $cumulative = FALSE) {
    $filter = ercore_get_filter_dates();
    // ! for checking dates
    if ($cumulative == TRUE) {
      
      $filter = ercore_get_project_filter_dates();
      //subtracting a day since Drupal dates are wonky.
      $filter['start']-=86400;
    }
    // dpm("------------ouptutDateFilter-------------");
    // dpm($cumulative, "cumulative?");
    // dpm("filter");
    // dpm(date("Y-m-d\TH:i:s\Z",$filter['start']));
    // dpm(date("Y-m-d\TH:i:s\Z",$filter['end']));
    // dpm("--");
    // dpm("date");
    // dpm(date("Y-m-d\TH:i:s\Z",$date));
    // dpm("-------------------------");
    if ($filter['start'] <= $date && $date <= $filter['end']) {
      return 1;
    }
    return 0;
  }

  /**
   * Filter Output Dates.
   *
   * @param array $dates
   *   Date to be verified.
   * @param bool $cumulative
   *   Use cumulative dates or not.
   *
   * @return int
   *   Return status of date in range.
   */
  public static function outputDateRangeFilter(array $dates, $cumulative = FALSE) {
    $filter = ercore_get_filter_dates();
    if ($cumulative === TRUE) {
      $filter = ercore_get_project_filter_dates();
    }
    // dpm("------------------------ouptutDateRangeFilter--------------------");
    // dpm("filter");
    // dpm(date("Y-m-d\TH:i:s\Z",$filter['start']), "filter['start']");
    // dpm(date("Y-m-d\TH:i:s\Z",$filter['end']), "filter['end']");
    // dpm("--");
    // dpm("dates");
    // dpm(date("Y-m-d\TH:i:s\Z",$dates['start']), "dates['start']");
    // dpm(date("Y-m-d\TH:i:s\Z",$dates['end']), "dates['end']");
    // dpm("----------------------------------------------------------------");
    
    if (($dates['start'] <= $filter['end'] && $dates['end'] >= $filter['start'])
      || ($dates['start'] <= $filter['end'] && $dates['start'] >= $filter['start'] && empty($dates['end']))
    ) {
      return 1;
    }
    return 0;
  }

  /**
   * Get processed data.
   */
  // public static function getData() {
  //   $data = self::buildDataArray();
  //   $data = self::filteredPatents($data);
  //   $data = self::filteredProposals($data);
  //   $data = self::filteredPublications($data);
  //   $data = self::filteredUsers($data, 'hired');
  //   $data = self::filteredUsers($data, 'post-doc');
  //   $data = self::filteredUsers($data, 'graduate');
  //   $data = self::filteredUsers($data, 'undergraduate');
  //   return $data;
  // }

}
