<?php

namespace Drupal\ercore_core\Form;

/**
 * @file
 * Contains Drupal\ercore\Form\ERCoreTableE.
 */

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\ercore_core\ErcoreOutputs;
use Drupal\Core\Link;
use Drupal\Core\Url;

/**
 * Class ERCoreTableE.
 *
 * Defines ERCore Table E.
 *
 * @package Drupal\ercore\Form
 */
class ERCoreTableE extends FormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'ERCoreTableE';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $url = Url::fromRoute('ercore_core.outputs_export');
    $link = Link::fromTextAndUrl('Download NSF Table E.', $url);
    $form['#attached']['library'][] = 'ercore_core/ercore-core-exports.library';
    $form['date_filter'] = \Drupal::formBuilder()
      ->getForm('Drupal\ercore_core\Form\ERCoreDateFilter');
    $form['data_table'] = [
      '#type' => 'fieldset',
      '#title' => t('Table E Results'),
      '#open' => TRUE,
    ];
    $form['data_table']['description'] = self::formatResults();
    $form['export_link'] = [
      '#markup' => '<p class="epscor-download">' . $link->toString() . '</p>',
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    // We don't use this, but the interface requires us to implement it.
  }

  /**
   * Format Results.
   */
  public function formatResults() {
    $rows = ErcoreOutputs::getData();
    // Patents.
    $patents = [
      'awarded' => [
        [
          'data' => 'Awarded',
          'header' => TRUE,
        ],
        $rows['patents']['awarded']['current'] > 0 ? t('<a class="atagcircle" href="/ercore/admin-views/outputs_expanded?contenttype=patents&type=awarded&cumulative=false">'.$rows['patents']['awarded']['current'].'</a>') : $rows['patents']['awarded']['current'],
        $rows['patents']['awarded']['cumulative']  > 0 ? t('<a class="atagcircle" href="/ercore/admin-views/outputs_expanded?contenttype=patents&type=awarded&cumulative=true">'.$rows['patents']['awarded']['cumulative'].'</a>') : $rows['patents']['awarded']['current'],
      ],
      'pending' => [
        [
          'data' => 'Pending',
          'header' => TRUE,
        ],
        $rows['patents']['pending']['current'] > 0 ? t('<a class="atagcircle" href="/ercore/admin-views/outputs_expanded?contenttype=patents&type=pending&cumulative=false">'.$rows['patents']['pending']['current'].'</a>') : $rows['patents']['pending']['current'],
        $rows['patents']['pending']['cumulative']  > 0 ? t('<a class="atagcircle" href="/ercore/admin-views/outputs_expanded?contenttype=patents&type=pending&cumulative=true">'.$rows['patents']['pending']['cumulative'].'</a>') : $rows['patents']['pending']['current'],
      ],
      'licensed' => [
        [
          'data' => 'Licensed',
          'header' => TRUE,
        ],
        $rows['patents']['licensed']['current'] > 0 ? t('<a class="atagcircle" href="/ercore/admin-views/outputs_expanded?contenttype=patents&type=licensed&cumulative=false">'.$rows['patents']['licensed']['current'].'</a>') : $rows['patents']['licensed']['current'],
        $rows['patents']['licensed']['cumulative']  > 0 ? t('<a class="atagcircle" href="/ercore/admin-views/outputs_expanded?contenttype=patents&type=licensed&cumulative=true">'.$rows['patents']['licensed']['cumulative'].'</a>') : $rows['patents']['licensed']['current'],
      ],
    ];
    $results['patents'] = [
      '#theme' => 'table',
      '#caption' => 'Patents',
      '#header' => [
        'Category',
        'Total for Current Reporting Period',
        'Cumulative Total for the Award',
      ],
      '#rows' => $patents,
      '#attributes' => [
        'class' => [
          'ercore-data-table',
        ],
      ],
    ];
    // Proposals.
    $proposals = [
      'submitted' => [
        [
          'data' => 'Submitted',
          'header' => TRUE,
        ],
        $rows['proposals']['submitted']['current']['number']> 0 ? t('<a class="atagcircle" href="/ercore/admin-views/outputs_expanded?contenttype=proposals_grants&type=submitted&cumulative=false">'.$rows['proposals']['submitted']['current']['number'].'</a>') : $rows['proposals']['submitted']['current']['number'],
        $rows['proposals']['submitted']['current']['funds'],
        0,
        $rows['proposals']['submitted']['cumulative']['number']> 0 ? t('<a class="atagcircle" href="/ercore/admin-views/outputs_expanded?contenttype=proposals_grants&type=submitted&cumulative=true">'.$rows['proposals']['submitted']['cumulative']['number'].'</a>') : $rows['proposals']['submitted']['cumulative']['number'],
        $rows['proposals']['submitted']['cumulative']['funds'],
        0,
      ],
      'awarded' => [
        [
          'data' => 'Awarded',
          'header' => TRUE,
        ],
        $rows['proposals']['awarded']['current']['number']> 0 ? t('<a class="atagcircle" href="/ercore/admin-views/outputs_expanded?contenttype=proposals_grants&type=awarded&cumulative=false">'.$rows['proposals']['awarded']['current']['number'].'</a>') : $rows['proposals']['awarded']['current']['number'],
        $rows['proposals']['awarded']['current']['funds'],
        $rows['proposals']['awarded']['current']['awarded_funds'],
        $rows['proposals']['awarded']['cumulative']['number']> 0 ? t('<a class="atagcircle" href="/ercore/admin-views/outputs_expanded?contenttype=proposals_grants&type=awarded&cumulative=true">'.$rows['proposals']['awarded']['cumulative']['number'].'</a>') : $rows['proposals']['awarded']['cumulative']['number'],
        $rows['proposals']['awarded']['cumulative']['funds'],
        $rows['proposals']['awarded']['cumulative']['awarded_funds'],
      ],
      'pending' => [
        [
          'data' => 'Pending',
          'header' => TRUE,
        ],
        $rows['proposals']['pending']['current']['number']> 0 ? t('<a class="atagcircle" href="/ercore/admin-views/outputs_expanded?contenttype=proposals_grants&type=pending&cumulative=false">'.$rows['proposals']['pending']['current']['number'].'</a>') : $rows['proposals']['pending']['current']['number'],
        $rows['proposals']['pending']['current']['funds'],
        0,
        $rows['proposals']['pending']['cumulative']['number']> 0 ? t('<a class="atagcircle" href="/ercore/admin-views/outputs_expanded?contenttype=proposals_grants&type=pending&cumulative=true">'.$rows['proposals']['pending']['cumulative']['number'].'</a>') : $rows['proposals']['pending']['cumulative']['number'],
        $rows['proposals']['pending']['cumulative']['funds'],
        0,
      ],
    ];
    $results['proposals'] = [
      '#theme' => 'table',
      '#caption' => 'Proposals / Grants / Contracts',
      '#header' => [
        'Category',
        'Current Number',
        'Current Funds Requested',
        'Current Funds Awarded',
        'Cumulative Number',
        'Cumulative Funds Requested',
        'Cumulative Funds Awarded',
      ],
      '#rows' => $proposals,
      '#attributes' => [
        'class' => [
          'ercore-data-table',
        ],
      ],
    ];
    // Publications.
    $publications = [
      'primary' => [
        [
          'data' => 'Primary RII Support',
          'header' => TRUE,
        ],
        $rows['publications']['primary']['current']> 0 ? t('<a class="atagcircle" href="/ercore/admin-views/outputs_expanded?contenttype=publications&type=primary&cumulative=false">'.$rows['publications']['primary']['current'].'</a>') : $rows['publications']['primary']['current'],
        $rows['publications']['primary']['cumulative']> 0 ? t('<a class="atagcircle" href="/ercore/admin-views/outputs_expanded?contenttype=publications&type=primary&cumulative=true">'.$rows['publications']['primary']['cumulative'].'</a>') : $rows['publications']['primary']['cumulative'],
      ],
      'partial' => [
        [
          'data' => 'Partial RII Support',
          'header' => TRUE,
        ],
        $rows['publications']['partial']['current']> 0 ? t('<a class="atagcircle" href="/ercore/admin-views/outputs_expanded?contenttype=publications&type=partial&cumulative=false">'.$rows['publications']['partial']['current'].'</a>') : $rows['publications']['partial']['current'],
        $rows['publications']['partial']['cumulative']> 0 ? t('<a class="atagcircle" href="/ercore/admin-views/outputs_expanded?contenttype=publications&type=partial&cumulative=true">'.$rows['publications']['partial']['cumulative'].'</a>') : $rows['publications']['partial']['cumulative'],
      ],
    ];
    $results['publications'] = [
      '#theme' => 'table',
      '#caption' => 'Published Publications',
      '#header' => [
        'Category',
        'Total for Current Reporting Period',
        'Cumulative Total for the Award',
      ],
      '#rows' => $publications,
      '#attributes' => [
        'class' => [
          'ercore-data-table',
        ],
      ],
    ];
    // New Hires.
    $hired = [
      'male' => [
        [
          'data' => 'Male',
          'header' => TRUE,
        ],
        $rows['hired']['male']['current']> 0 ? t('<a class="atagcircle" href="/ercore/admin-views/outputs_expanded?contenttype=hired&type=male&cumulative=false">'.$rows['hired']['male']['current'].'</a>') : $rows['hired']['male']['current'],
        $rows['hired']['male']['cumulative']> 0 ? t('<a class="atagcircle" href="/ercore/admin-views/outputs_expanded?contenttype=hired&type=male&cumulative=true">'.$rows['hired']['male']['cumulative'].'</a>') : $rows['hired']['male']['cumulative'],
      ],
      'female' => [
        [
          'data' => 'Female',
          'header' => TRUE,
        ],
        $rows['hired']['female']['current']> 0 ? t('<a class="atagcircle" href="/ercore/admin-views/outputs_expanded?contenttype=hired&type=female&cumulative=false">'.$rows['hired']['female']['current'].'</a>') : $rows['hired']['female']['current'],
        $rows['hired']['female']['cumulative']> 0 ? t('<a class="atagcircle" href="/ercore/admin-views/outputs_expanded?contenttype=hired&type=female&cumulative=true">'.$rows['hired']['female']['cumulative'].'</a>') : $rows['hired']['female']['cumulative'],

      ],
      'minority' => [
        [
          'data' => 'Underrepresented Minority',
          'header' => TRUE,
        ],
        $rows['hired']['minority']['current']> 0 ? t('<a class="atagcircle" href="/ercore/admin-views/outputs_expanded?contenttype=hired&type=minority&cumulative=false">'.$rows['hired']['minority']['current'].'</a>') : $rows['hired']['minority']['current'],
        $rows['hired']['minority']['cumulative']> 0 ? t('<a class="atagcircle" href="/ercore/admin-views/outputs_expanded?contenttype=hired&type=minority&cumulative=true">'.$rows['hired']['minority']['cumulative'].'</a>') : $rows['hired']['minority']['cumulative'],
      ],
      'disabled' => [
        [
          'data' => 'Disabled',
          'header' => TRUE,
        ],
        $rows['hired']['disabled']['current']> 0 ? t('<a class="atagcircle" href="/ercore/admin-views/outputs_expanded?contenttype=hired&type=disabled&cumulative=false">'.$rows['hired']['disabled']['current'].'</a>') : $rows['hired']['disabled']['current'],
        $rows['hired']['disabled']['cumulative']> 0 ? t('<a class="atagcircle" href="/ercore/admin-views/outputs_expanded?contenttype=hired&type=disabled&cumulative=true">'.$rows['hired']['disabled']['cumulative'].'</a>') : $rows['hired']['disabled']['cumulative'],
      ],
    ];
    $results['new_hires'] = [
      '#theme' => 'table',
      '#caption' => 'Total New Faculty Hired',
      '#header' => [
        'Category',
        'Total for Current Reporting Period',
        'Cumulative Total for the Award',

      ],
      '#rows' => $hired,
      '#attributes' => [
        'class' => [
          'ercore-data-table',
        ],
      ],
    ];
    // Post Docs.
    $postdoc = [
      'male' => [
        [
          'data' => 'Male',
          'header' => TRUE,
        ],
        // $rows['post-doc']['male']['current'],
        // $rows['post-doc']['male']['cumulative'],
        $rows['post-doc']['male']['current']> 0 ? t('<a class="atagcircle" href="/ercore/admin-views/outputs_expanded?contenttype=post-doc&type=male&cumulative=false">'.$rows['post-doc']['male']['current'].'</a>') : $rows['post-doc']['male']['current'],
        $rows['post-doc']['male']['cumulative']> 0 ? t('<a class="atagcircle" href="/ercore/admin-views/outputs_expanded?contenttype=post-doc&type=male&cumulative=true">'.$rows['post-doc']['male']['cumulative'].'</a>') : $rows['post-doc']['male']['cumulative'],
      ],
      'female' => [
        [
          'data' => 'Female',
          'header' => TRUE,
        ],
        // $rows['post-doc']['female']['current'],
        // $rows['post-doc']['female']['cumulative'],
        $rows['post-doc']['female']['current']> 0 ? t('<a class="atagcircle" href="/ercore/admin-views/outputs_expanded?contenttype=post-doc&type=female&cumulative=false">'.$rows['post-doc']['female']['current'].'</a>') : $rows['post-doc']['female']['current'],
        $rows['post-doc']['female']['cumulative']> 0 ? t('<a class="atagcircle" href="/ercore/admin-views/outputs_expanded?contenttype=post-doc&type=female&cumulative=true">'.$rows['post-doc']['female']['cumulative'].'</a>') : $rows['post-doc']['female']['cumulative'],

      ],
      'minority' => [
        [
          'data' => 'Underrepresented Minority',
          'header' => TRUE,
        ],
        // $rows['post-doc']['minority']['current'],
        // $rows['post-doc']['minority']['cumulative'],
        $rows['post-doc']['minority']['current']> 0 ? t('<a class="atagcircle" href="/ercore/admin-views/outputs_expanded?contenttype=post-doc&type=minority&cumulative=false">'.$rows['post-doc']['minority']['current'].'</a>') : $rows['post-doc']['minority']['current'],
        $rows['post-doc']['minority']['cumulative']> 0 ? t('<a class="atagcircle" href="/ercore/admin-views/outputs_expanded?contenttype=post-doc&type=minority&cumulative=true">'.$rows['post-doc']['minority']['cumulative'].'</a>') : $rows['post-doc']['minority']['cumulative'],
      ],
      'disabled' => [
        [
          'data' => 'Disabled',
          'header' => TRUE,
        ],
        // $rows['post-doc']['disabled']['current'],
        // $rows['post-doc']['disabled']['cumulative'],
        $rows['post-doc']['disabled']['current']> 0 ? t('<a class="atagcircle" href="/ercore/admin-views/outputs_expanded?contenttype=post-doc&type=disabled&cumulative=false">'.$rows['post-doc']['disabled']['current'].'</a>') : $rows['post-doc']['disabled']['current'],
        $rows['post-doc']['disabled']['cumulative']> 0 ? t('<a class="atagcircle" href="/ercore/admin-views/outputs_expanded?contenttype=post-doc&type=disabled&cumulative=true">'.$rows['post-doc']['disabled']['cumulative'].'</a>') : $rows['post-doc']['disabled']['cumulative'],
      ],
    ];
    $results['postdocs'] = [
      '#theme' => 'table',
      '#caption' => 'Total Post Docs Involved',
      '#header' => [
        'Category',
        'Total for Current Reporting Period',
        'Cumulative Total for the Award',
      ],
      '#rows' => $postdoc,
      '#attributes' => [
        'class' => [
          'ercore-data-table',
        ],
      ],
    ];
    // Graduate.
    $graduate = [
      'male' => [
        [
          'data' => 'Male',
          'header' => TRUE,
        ],
        $rows['graduate']['male']['current']> 0 ? t('<a class="atagcircle" href="/ercore/admin-views/outputs_expanded?contenttype=graduate&type=male&cumulative=false">'.$rows['graduate']['male']['current'].'</a>') : $rows['graduate']['male']['current'],
        $rows['graduate']['male']['cumulative']> 0 ? t('<a class="atagcircle" href="/ercore/admin-views/outputs_expanded?contenttype=graduate&type=male&cumulative=true">'.$rows['graduate']['male']['cumulative'].'</a>') : $rows['graduate']['male']['cumulative'],
        // $rows['graduate']['male']['current'],
        // $rows['graduate']['male']['cumulative'],
      ],
      'female' => [
        [
          'data' => 'Female',
          'header' => TRUE,
        ],
        $rows['graduate']['female']['current']> 0 ? t('<a class="atagcircle" href="/ercore/admin-views/outputs_expanded?contenttype=graduate&type=female&cumulative=false">'.$rows['graduate']['female']['current'].'</a>') : $rows['graduate']['female']['current'],
        $rows['graduate']['female']['cumulative']> 0 ? t('<a class="atagcircle" href="/ercore/admin-views/outputs_expanded?contenttype=graduate&type=female&cumulative=true">'.$rows['graduate']['female']['cumulative'].'</a>') : $rows['graduate']['female']['cumulative'],
        // $rows['graduate']['female']['current'],
        // $rows['graduate']['female']['cumulative'],

      ],
      'minority' => [
        [
          'data' => 'Underrepresented Minority',
          'header' => TRUE,
        ],
        $rows['graduate']['minority']['current']> 0 ? t('<a class="atagcircle" href="/ercore/admin-views/outputs_expanded?contenttype=graduate&type=minority&cumulative=false">'.$rows['graduate']['minority']['current'].'</a>') : $rows['graduate']['minority']['current'],
        $rows['graduate']['minority']['cumulative']> 0 ? t('<a class="atagcircle" href="/ercore/admin-views/outputs_expanded?contenttype=graduate&type=minority&cumulative=true">'.$rows['graduate']['minority']['cumulative'].'</a>') : $rows['graduate']['minority']['cumulative'],
        // $rows['graduate']['minority']['current'],
        // $rows['graduate']['minority']['cumulative'],
      ],
      'disabled' => [
        [
          'data' => 'Disabled',
          'header' => TRUE,
        ],
        $rows['graduate']['disabled']['current']> 0 ? t('<a class="atagcircle" href="/ercore/admin-views/outputs_expanded?contenttype=graduate&type=disabled&cumulative=false">'.$rows['graduate']['disabled']['current'].'</a>') : $rows['graduate']['disabled']['current'],
        $rows['graduate']['disabled']['cumulative']> 0 ? t('<a class="atagcircle" href="/ercore/admin-views/outputs_expanded?contenttype=graduate&type=disabled&cumulative=true">'.$rows['graduate']['disabled']['cumulative'].'</a>') : $rows['graduate']['disabled']['cumulative'],
        // $rows['graduate']['disabled']['current'],
        // $rows['graduate']['disabled']['cumulative'],
      ],
    ];
    $results['graduate'] = [
      '#theme' => 'table',
      '#caption' => 'Total Graduate Students Graduated',
      '#header' => [
        'Category',
        'Total for Current Reporting Period',
        'Cumulative Total for the Award',
      ],
      '#rows' => $graduate,
      '#attributes' => [
        'class' => [
          'ercore-data-table',
        ],
      ],
    ];
    // Undergraduate.
    $undergraduate = [
      'male' => [
        [
          'data' => 'Male',
          'header' => TRUE,
        ],
        // $rows['undergraduate']['male']['current'],
        // $rows['undergraduate']['male']['cumulative'],
        $rows['undergraduate']['male']['current']> 0 ? t('<a class="atagcircle" href="/ercore/admin-views/outputs_expanded?contenttype=undergraduate&type=male&cumulative=false">'.$rows['undergraduate']['male']['current'].'</a>') : $rows['undergraduate']['male']['current'],
        $rows['undergraduate']['male']['cumulative']> 0 ? t('<a class="atagcircle" href="/ercore/admin-views/outputs_expanded?contenttype=undergraduate&type=male&cumulative=true">'.$rows['undergraduate']['male']['cumulative'].'</a>') : $rows['undergraduate']['male']['cumulative'],
      ],
      'female' => [
        [
          'data' => 'Female',
          'header' => TRUE,
        ],
        $rows['undergraduate']['female']['current']> 0 ? t('<a class="atagcircle" href="/ercore/admin-views/outputs_expanded?contenttype=undergraduate&type=female&cumulative=false">'.$rows['undergraduate']['female']['current'].'</a>') : $rows['undergraduate']['female']['current'],
        $rows['undergraduate']['female']['cumulative']> 0 ? t('<a class="atagcircle" href="/ercore/admin-views/outputs_expanded?contenttype=undergraduate&type=female&cumulative=true">'.$rows['undergraduate']['female']['cumulative'].'</a>') : $rows['undergraduate']['female']['cumulative'],
        // $rows['undergraduate']['female']['current'],
        // $rows['undergraduate']['female']['cumulative'],

      ],
      'minority' => [
        [
          'data' => 'Underrepresented Minority',
          'header' => TRUE,
        ],
        $rows['undergraduate']['minority']['current']> 0 ? t('<a class="atagcircle" href="/ercore/admin-views/outputs_expanded?contenttype=undergraduate&type=minority&cumulative=false">'.$rows['undergraduate']['minority']['current'].'</a>') : $rows['undergraduate']['minority']['current'],
        $rows['undergraduate']['minority']['cumulative']> 0 ? t('<a class="atagcircle" href="/ercore/admin-views/outputs_expanded?contenttype=undergraduate&type=minority&cumulative=true">'.$rows['undergraduate']['minority']['cumulative'].'</a>') : $rows['undergraduate']['minority']['cumulative'],
        // $rows['undergraduate']['minority']['current'],
        // $rows['undergraduate']['minority']['cumulative'],
      ],
      'disabled' => [
        [
          'data' => 'Disabled',
          'header' => TRUE,
        ],
        $rows['undergraduate']['disabled']['current']> 0 ? t('<a class="atagcircle" href="/ercore/admin-views/outputs_expanded?contenttype=undergraduate&type=disabled&cumulative=false">'.$rows['undergraduate']['disabled']['current'].'</a>') : $rows['undergraduate']['disabled']['current'],
        $rows['undergraduate']['disabled']['cumulative']> 0 ? t('<a class="atagcircle" href="/ercore/admin-views/outputs_expanded?contenttype=undergraduate&type=disabled&cumulative=true">'.$rows['undergraduate']['disabled']['cumulative'].'</a>') : $rows['undergraduate']['disabled']['cumulative'],
        // $rows['undergraduate']['disabled']['current'],
        // $rows['undergraduate']['disabled']['cumulative'],
      ],
    ];
    $results['undergraduate'] = [
      '#theme' => 'table',
      '#caption' => 'Total Undergraduates Graduated',
      '#header' => [
        'Category',
        'Total for Current Reporting Period',
        'Cumulative Total for the Award',
      ],
      '#rows' => $undergraduate,
      '#attributes' => [
        'class' => [
          'ercore-data-table',
        ],
      ],
    ];
    return $results;
  }

}
